{-# OPTIONS_GHC -Wno-orphans #-}
{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE MultiWayIf #-}

module Gargantext.Orphans.Accelerate where

import Prelude
import Test.QuickCheck
import Data.Array.Accelerate (DIM0, DIM1, DIM2, DIM3, Z (..), (:.) (..), Array, Elt, fromList, use, arrayShape)
import Data.Array.Accelerate qualified as A
import qualified Data.Array.Accelerate.Sugar.Shape as AS
import qualified Data.Array.Accelerate.Interpreter as Naive

instance Arbitrary DIM0 where
  arbitrary = return Z

instance Arbitrary DIM1 where
  arbitrary = (Z :.) <$> choose (0,1024)
  shrink = \(Z :. i) -> if i <= 0 then [] else [Z :. i - 1 ]

instance Arbitrary DIM2 where
  arbitrary = do
    x <- choose (0,128)
    y <- choose (0,48)
    return (Z :. y :. x)
  shrink = \(Z :. r :. c) ->
    if | r <= 0 -> []
       | c <= 0 -> []
       | otherwise -> [Z :. (r - 1)
                         :. (c - 1)
                      ]

instance Arbitrary DIM3 where
  arbitrary = do
    x <- choose (0,64)
    y <- choose (0,32)
    z <- choose (0,16)
    return (Z :. z :. y :. x)

instance (Elt e, Arbitrary e) => Arbitrary (Array DIM2 e) where
  arbitrary = do
    sh <- arbitrary
    fromList sh <$> vectorOf (AS.size sh) arbitrary
  shrink arr = sliceArray arr

-- Slice the array to the new shape, keeping the square dimensions.
sliceArray :: Elt e => Array DIM2 e -> [Array DIM2 e]
sliceArray arr =
  case arrayShape arr of
    (Z :. x :. y) -> case (x, y) of
      (0,0) -> []
      _     -> [ Naive.run $ A.init $ A.transpose $ A.init $ use arr ]
